/*--------------------------------------------------------------------
 * FICHERO:  AhoraEs.c
 * OBJETIVO: Mostrar en una ventana la fecha y la hora y eliminar
 *           la ventana en cuanto se pulse una tecla o el ratn
 * AUTOR:    Pedro Reina
 * FECHA:    M.2.3.1999
 *------------------------------------------------------------------*/

/*--------------------------------------------------------------------
 * Ficheros de cabecera
 *------------------------------------------------------------------*/

#include <Windows.h>
#include <dos.h>     // struct date  getdate()  struct time  gettime()
#include <stdio.h>   // sprintf()
#include <string.h>  // strlen()

/*--------------------------------------------------------------------
 * Definicin de tipos
 *------------------------------------------------------------------*/

typedef long int entero;

/*--------------------------------------------------------------------
 * Declaracin de funciones
 *------------------------------------------------------------------*/

LONG FAR PASCAL ProcedimientoPrincipal (HWND,UINT,WPARAM,LPARAM);
BOOL CentraVentana (HWND);
void Escribe (HWND,HDC);
entero DiaSemana(entero,entero,entero);

/*--------------------------------------------------------------------
 * Definicin de macros constantes
 *------------------------------------------------------------------*/

#define ID_CRONO 101

/*--------------------------------------------------------------------
 * Definicin de macros funciones
 *------------------------------------------------------------------*/

/*--------------------------------------------------------------------
 * MACRO:    AnchoPantalla()
 * OBJETIVO: Calcular el ancho total de la pantalla
 *------------------------------------------------------------------*/
#define AnchoPantalla()  GetSystemMetrics(SM_CXSCREEN)

/*--------------------------------------------------------------------
 * MACRO:    AltoPantalla()
 * OBJETIVO: Calcular el alto total de la pantalla
 *------------------------------------------------------------------*/
#define AltoPantalla()  GetSystemMetrics(SM_CYSCREEN)

/*--------------------------------------------------------------------
 * Definicin de variables globales
 *------------------------------------------------------------------*/

// Ninguna

/*--------------------------------------------------------------------
 * Programa principal
 *------------------------------------------------------------------*/
#pragma argsused
int PASCAL
WinMain
  (
  HANDLE Instancia,    // Manejador de instancia de aplicacin
  HANDLE Anterior,     // Manejador de instancia anterior
  LPSTR  Parametros,   // Cadena con la lnea de parmetros
  int    ModoMostrar   // Aplicacin como ventana o como icono
  )
  {
  HWND     VentanaPrincipal;
  MSG      Mensaje;
  WNDCLASS Clase;
  HMENU    MenuControl;

  // Si es necesario, registramos la clase de ventana
  if ( !Anterior )
    {
    // Estilo de ventana
    Clase.style = CS_HREDRAW | CS_VREDRAW;

    // Funcin que maneja la ventana
    Clase.lpfnWndProc = ProcedimientoPrincipal;

    // Octetos extra
    Clase.cbClsExtra = 0;

    // Octetos extra
    Clase.cbWndExtra = DLGWINDOWEXTRA;

    // Instancia de aplicacin
    Clase.hInstance = Instancia;

    // Icono al minimizar
    Clase.hIcon = LoadIcon (Instancia,"AhoraEs");

    // Cursor
    Clase.hCursor = LoadCursor (NULL,IDC_ARROW);

    // Pincel de fondo
    Clase.hbrBackground = COLOR_WINDOW + 1;

    // Recurso de men
    Clase.lpszMenuName = NULL;

    // Nombre de la clase de ventana
    Clase.lpszClassName = "AhoraEs";

    if ( !RegisterClass (&Clase) )
      { return ( FALSE ); }
    }

  // Creamos la ventana principal
  VentanaPrincipal = CreateDialog (Instancia, "Principal", 0, NULL);

  // Cambiamos el men del cuadro de control
  MenuControl = GetSystemMenu (VentanaPrincipal, FALSE);
  DeleteMenu (MenuControl, SC_SIZE,     MF_BYCOMMAND);
  DeleteMenu (MenuControl, SC_MAXIMIZE, MF_BYCOMMAND);

  // Mostramos la ventana
  CentraVentana (VentanaPrincipal);
  ShowWindow (VentanaPrincipal, ModoMostrar);

  // Manejamos los mensajes
  while ( GetMessage (&Mensaje, NULL, NULL, NULL) )
    {
    TranslateMessage (&Mensaje);
    DispatchMessage (&Mensaje);
    }

  return ( Mensaje.wParam );
  }

/*--------------------------------------------------------------------
 * Definicin de funciones
 *------------------------------------------------------------------*/

/*--------------------------------------------------------------------
 * FUNCION:  ProcedimientoPrincipal()
 * OBJETIVO: Recibir los mensajes de Windows para la clase
 *           de ventanas principal de esta aplicacin
 *------------------------------------------------------------------*/
LONG FAR PASCAL
ProcedimientoPrincipal
  (
  HWND   Ventana,        // Ventana a la que se enva el mensaje
  UINT   Mensaje,        // El mensaje
  WPARAM ParametroCorto, // Informacin adicional
  LPARAM ParametroLargo  // Informacin adicional
  )
  {
  BOOL        Respuesta = FALSE;
  PAINTSTRUCT Pintura;
  HDC         Contexto;
  UINT        Cronometro;

  switch ( Mensaje )
    {
    case WM_CREATE:
      Cronometro = SetTimer (Ventana, ID_CRONO, 1000, NULL);
      break;

    case WM_TIMER:
      InvalidateRect (Ventana, NULL, TRUE);
      break;

    case WM_KEYDOWN:
    case WM_RBUTTONDOWN:
    case WM_LBUTTONDOWN:
      SendMessage (Ventana, WM_CLOSE, 0, 0L);
      break;

    case WM_PAINT:
      Contexto = BeginPaint (Ventana,&Pintura);
      Escribe (Ventana, Contexto);
      EndPaint (Ventana,&Pintura);
      break;

    case WM_CLOSE:
      SendMessage (Ventana, WM_DESTROY, 0, 0L);
      break;

    case WM_DESTROY:
      if ( Cronometro )  { KillTimer (Ventana, ID_CRONO); }
      PostQuitMessage (0);
      break;

    default:
      Respuesta = DefWindowProc (Ventana, Mensaje,
                                 ParametroCorto, ParametroLargo);
      break;
    }

  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  CentraVentana()
 * OBJETIVO: Centrar una ventana en pantalla
 * ENTRADAS: La ventana
 * SALIDAS:  Lgica, que indica si ha cambiado la posicin de la ventana
 * NOTA:     Slo se cambia la posicin
 *------------------------------------------------------------------*/
BOOL
CentraVentana
  (
  HWND Ventana
  )
  {
  BOOL Respuesta = FALSE;
  RECT Info;
  int  AnchoVentana, AltoVentana, Izquierda, Arriba;

  GetWindowRect (Ventana, &Info);

  AnchoVentana = Info.right - Info.left;
  AltoVentana = Info.bottom - Info.top;

  Izquierda = (AnchoPantalla()-AnchoVentana) / 2;
  Arriba = (AltoPantalla()-AltoVentana) / 2;

  if ( Izquierda >= 0 && Arriba >= 0 )
    {
    SetWindowPos (Ventana, HWND_TOP, Izquierda, Arriba, 0, 0,
                  SWP_NOACTIVATE | SWP_NOSIZE | SWP_NOREDRAW | SWP_NOZORDER);
    Respuesta = TRUE;
    }

  return ( Respuesta );
  }
/*--------------------------------------------------------------------
 * FUNCION:  Escribe()
 * OBJETIVO: Escribir la fecha y la hora en una ventana
 * ENTRADAS: La ventana y un contexto para dibujar
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void Escribe
  (
  HWND Ventana,
  HDC  Contexto
  )
  {
  char * Nombre = "DLMXJVS";
  char Texto[20];

  RECT Info;
  TEXTMETRIC Metrica;

  int Centro, Alto;

  struct date d;
  struct time t;

  getdate(&d);
  gettime(&t);

  // Calculamos las medidas que vamos a usar
  GetClientRect (Ventana, &Info);
  Centro = Info.right/2;
  GetTextMetrics (Contexto, &Metrica);
  Alto = Metrica.tmHeight;

  // Ajustamos el modo de presentar texto
  SetTextAlign (Contexto, TA_CENTER);

  // Usamos los colores del sistema
  SetTextColor (Contexto, GetSysColor(COLOR_WINDOWTEXT));
  SetBkColor (Contexto, GetSysColor(COLOR_WINDOW));

  // Escribimos la fecha
  sprintf (Texto,"%c.%d.%d.%d",
           Nombre [DiaSemana (d.da_year, d.da_mon, d.da_day)],
           d.da_day, d.da_mon, d.da_year);
  TextOut (Contexto, Centro, Alto/2, Texto, strlen(Texto));

  // Escribimos la hora
  sprintf (Texto,"%02d:%02d:%02d", t.ti_hour, t.ti_min, t.ti_sec);
  TextOut (Contexto, Centro, 2*Alto, Texto, strlen(Texto));
  }

/*--------------------------------------------------------------------
 * FUNCION:  DiaSemana()
 * OBJETIVO: Decir el da de la semana de una fecha
 * ENTRADAS: El ao, el mes y el da
 * SALIDAS:  Un entero que indica el da de la semana, con este convenio:
 *           0 -> Domingo, 1 -> Lunes, ..., 6 -> Sbado
 * EJEMPLO:  DiaSemana (1994,10,31)
 *------------------------------------------------------------------*/
entero DiaSemana (entero Ano, entero Mes, entero Dia)
  {
  entero Sem, Factor;

  Factor = 365 * Ano + Dia + 31*(Mes-1);
  if ( Mes < 3 ) { Ano--; }
  Factor += (entero)(Ano/4.0) - (entero)(.75*((entero)(Ano/100.0)+1));
  if ( Mes >= 3 ) { Factor -= (entero)(.4*Mes + 2.3); }
  Sem = Factor - 7*(entero)(Factor/7) - 1;
  if ( Sem == -1 ) { Sem = 6; }
  return ( Sem );
  }